/*
    ChibiOS - Copyright (C) 2006..2018 Giovanni Di Sirio

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/

#ifndef _BOARD_H_
#define _BOARD_H_

/*
 * Setup for the LeafLabs Maple Mini.
 */

/*
 * Board identifier.
 */
#define BOARD_MAPLEMINI_STM32_F103
#define BOARD_NAME              "BluePill"

/*
 * Board frequencies.
 */
#define STM32_LSECLK            32768
#define STM32_HSECLK            8000000

/*
 * MCU type, supported types are defined in ./os/hal/platforms/hal_lld.h.
 */
#define STM32F103xB

/*
 * IO pins assignments
 */

/* on-board */

#define GPIOC_LED     13
#define GPIOA_RELAY		0
#define GPIOA_SSR		  1

#define LCD_LED_ON		palClearLine(LINE_LCD_DC)
#define LCD_LED_OFF		palSetLine(LINE_LCD_DC)

#define LINE_LED		  PAL_LINE(GPIOC, 13U)
#define LINE_RELAY	  PAL_LINE(GPIOA, 0U)
#define LINE_SSR		  PAL_LINE(GPIOA, 1U)

#define LINE_STAB_OFF	    PAL_LINE(GPIOA, 2U)
#define LINE_RAZGON_OFF	  PAL_LINE(GPIOA, 3U)

/* Buttons */
#define LINE_BTN_4	  PAL_LINE(GPIOA, 8U)
#define LINE_BTN_3    PAL_LINE(GPIOB, 15U)
#define LINE_BTN_2    PAL_LINE(GPIOB, 14U)
#define LINE_BTN_1    PAL_LINE(GPIOB, 13U)
#define LINE_BTN_G    PAL_LINE(GPIOB, 12U)

#define BTNS_ON			  palClearLine(LINE_BTN_G)
#define BTNS_OFF		  palSetLine(LINE_BTN_G)

/* LCD */
#define LINE_LED		  PAL_LINE(GPIOC, 13U)
#define LINE_LCD_CS		PAL_LINE(GPIOA, 4U)
#define LINE_LCD_DC		PAL_LINE(GPIOA, 6U)

#define LCD_DC_CMD		palClearLine(LINE_LCD_DC)
#define LCD_DC_DATA		palSetLine(LINE_LCD_DC)
#define LCD_CS_RES		palClearLine(LINE_LCD_CS)
#define LCD_CS_SET		palSetLine(LINE_LCD_CS)

/* J1 connector */

// pin 1:       VBAT - Coin Cell 3V
#define GPIOC_TAMPER_RTC        13      // pin 2
#define GPIOC_OSC32_IN          14      // pin 3
#define GPIOC_OSD32_OUT         15      // pin 4
#define GPIOA_USART2_CTS        0       // pin 5  RELAY
#define GPIOA_USART2_RTS        1       // pin 6  SSR
#define GPIOA_USART2_TX         2       // pin 7  STAB_OFF
#define GPIOA_USART2_RX         3       // pin 8  RAZGON_OFF
#define GPIOA_SPI1_NSS          4       // pin 9  LCD_CS
#define GPIOA_SPI1_SCK          5       // pin 10 LCD_SCL
#define GPIOA_SPI1_MISO         6       // pin 11 LCD_DC
#define GPIOA_SPI1_MOSI         7       // pin 12 LCD_SDA
#define GPIOB_ADC12_IN8         0       // pin 13
#define GPIOB_ADC12_IN9         1       // pin 14 ADC_IN
#define GPIOB_I2C2_SCL          10      // pin 15
#define GPIOB_I2C2_SDA          11      // pin 16
// pin 17:       RESET
// pin 18:       3V3
// pin 19:       GND
// pin 20:       GND

/* J2 connector */

// pin 1:        3V3
// pin 2:        GND
// pin 3:        5V
#define GPIOB_PIN9              9       // pin 4  TIM4_CH4/I2C1_SDA/CANTX
#define GPIOB_PIN8              8       // pin 5  TIM4_CH3/I2C1_SCL/CANRX
#define GPIOB_PIN7              7       // pin 6  I2C1_SDA/TIM4_CH2/USART1_RX
#define GPIOB_PIN6              6       // pin 7  I2C1_SCL/TIM4_CH1/USART1_TX
#define GPIOB_PIN5              5       // pin 8  I2C1_SMBA/TIM3_CH2/SPI1_MOSI
#define GPIOB_JTRST             4       // pin 9  TIM3_CH1/PB4/SPI1_MISO
#define GPIOB_JTDO              3       // pin 10 TIM2_CH2/PB3/TRACESWO/SPI1_SCK
#define GPIOA_JTDI              15      // pin 11
#define GPIOA_USBDP             12      // pin 12
#define GPIOA_USBDM             11      // pin 13
#define GPIOA_USART1_RX         10      // pin 14
#define GPIOA_USART1_TX         9       // pin 15
#define GPIOA_USART1_CK         8       // pin 16 BTN_4
#define GPIOB_SPI2_MOSI         15      // pin 17 BTN_3
#define GPIOB_SPI2_MISO         14      // pin 18 BTN_2
#define GPIOB_SPI2_SCK          13      // pin 19 BTN_1
#define GPIOB_SPI2_NSS          12      // pin 20 BTN_GND

/*
 * I/O ports initial setup, this configuration is established soon after reset
 * in the initialization code.
 *
 * The digits have the following meaning:
 *   0 - Analog input.
 *   1 - Push Pull output 10MHz.
 *   2 - Push Pull output 2MHz.
 *   3 - Push Pull output 50MHz.
 *   4 - Digital input.
 *   5 - Open Drain output 10MHz.
 *   6 - Open Drain output 2MHz.
 *   7 - Open Drain output 50MHz.
 *   8 - Digital input with PullUp or PullDown resistor depending on ODR.
 *   9 - Alternate Push Pull output 10MHz.
 *   A - Alternate Push Pull output 2MHz.
 *   B - Alternate Push Pull output 50MHz.
 *   C - Reserved.
 *   D - Alternate Open Drain output 10MHz.
 *   E - Alternate Open Drain output 2MHz.
 *   F - Alternate Open Drain output 50MHz.
 * Please refer to the STM32 Reference Manual for details.
 */

/*
 * Port A setup.
 * Everything input with pull-up except:
 * PA0  - Push Pull output   (LCD_RELAY).
 * PA1  - Push Pull output   (LCD_SSR).
 * PA2  - Input with Pull-Up (STAB_OFF)
 * PA3  - Input with Pull-Up (RAZGON_OFF)
 * PA4  - Push Pull output   (LCD_CS).
 * PA5  - Alternate output   (LCD_SCL - SPI1 SCK).
 * PA6  - Push Pull output   (LCD_DC).
 * PA7  - Alternate output   (LCD_DSA - SPI1 MOSI).
 * PA8  - Input with Pull-Up (BTN_1)
 */
#define VAL_GPIOACRL            0xB3B38833      /*  PA7...PA0 */
#define VAL_GPIOACRH            0x88888888      /* PA15...PA8 */
#define VAL_GPIOAODR            0xFFFFFFFF

/*
 * Port B setup.
 * Everything input with pull-up except:
 * PB1  - Analog Input       (ADC_IN)
 * PB15 - Input with Pull-Up (BTN_2)
 * PB14 - Input with Pull-Up (BTN_3)
 * PB13 - Input with Pull-Up (BTN_4)
 * PB12 - Push Pull output   (BTN_G)
 */
#define VAL_GPIOBCRL            0x88888808      /*  PB7...PB0 */
#define VAL_GPIOBCRH            0x88838888      /* PB15...PB8 */
#define VAL_GPIOBODR            0xFFFFFFFF

/*
 * Port C setup.
 * Everything input with pull-up except:
 * PC13   - Push Pull output  (LED).
 */
#define VAL_GPIOCCRL            0x88888888      /*  PC7...PC0 */
#define VAL_GPIOCCRH            0x88288888      /* PC15...PC8 */
#define VAL_GPIOCODR            0xFFFFFFFF

/*
 * Port D setup.
 * Everything input with pull-up except:
 * PD0  - Normal input (XTAL).
 * PD1  - Normal input (XTAL).
 */
#define VAL_GPIODCRL            0x88888844      /*  PD7...PD0 */
#define VAL_GPIODCRH            0x88888888      /* PD15...PD8 */
#define VAL_GPIODODR            0xFFFFFFFF

/*
 * Port E setup.
 * Everything input with pull-up except:
 */
#define VAL_GPIOECRL            0x88888888      /*  PE7...PE0 */
#define VAL_GPIOECRH            0x88888888      /* PE15...PE8 */
#define VAL_GPIOEODR            0xFFFFFFFF

/*
 * USB bus activation macro, required by the USB driver.
 */
#define usb_lld_connect_bus(usbp) palClearPad(GPIOB, GPIOB_USB_DISC)

/*
 * USB bus de-activation macro, required by the USB driver.
 */
#define usb_lld_disconnect_bus(usbp) palSetPad(GPIOB, GPIOB_USB_DISC)

#if !defined(_FROM_ASM_)
#ifdef __cplusplus
extern "C" {
#endif
  void boardInit(void);
#ifdef __cplusplus
}
#endif
#endif /* _FROM_ASM_ */

#endif /* _BOARD_H_ */
